/*
 * Copyright (c) 2013-2023 NIBIO.
 *
 * This file is part of VIPSWeb 
 * (see https://gitlab.nibio.no/VIPS/VIPSWeb).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * Self-serving application that calculates flowering period for oat.
 * Depending on:
 * <ul>
 * 	<li>weather data source</li>
 * 	<li>the oat flowering model in the VIPS forecasting system </li>
 * 	<li>moment.js and moment-timezone.js (http://momentjs.com)</li>
 * </ul>
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */

// *********** Configuration section ****************

// Internal ID for the form
var theFormId = "_oatFloweringModelForm";
var detailURL = null;
var DEBUG = false;

// Settings for VIPS forecasting system
var serverUri = "https://coremanager.vips.nibio.no";
//var serverUri = "http://vipscoremanager";
var runUri= serverUri + "/models/OATFLOWERM/run"; 
var coreUserName = "";
var corePass = "";

// Location of weather data source
var weatherDataBaseUri = "http://lmt.nibio.no/agrometbase/export/getSeasonDailyTemperaturesJSON.php";
//var weatherDataBaseUri = "http://agrometbase-local/agrometbase/export/getSeasonDailyTemperaturesJSON.php";

// Setting current time
var now = moment();
//var now = moment("2014-07-04");


// ************ End configuration section ************

/**
 * Initializes the application. Sets up form, based on given options
 */
oatForm = function(options)
{
	// Get options, validate
	this.target = options.target;
	this.weatherStations = options.weatherStations;
	coreUsername = options.coreUserName;
	corePass = options.corePass;
	//console.log(options.now);
	now = options.now != undefined ? moment(options.now) : now;
	
	
	// Start stuff
	var workspace = document.getElementById(this.target);
	var height = workspace.offsetHeight;
	var width = workspace.offsetWidth;
	DEBUG = options.debug != null ? options.debug : false;
	workspace.innerHTML = getStartHTML();
	
	var theForm = document.getElementById(theFormId);
	
	// Then, add weather stations
	for(i in this.weatherStations)
	{
		var option = new Option(this.weatherStations[i].name,this.weatherStations[i].weatherstation_id);
		theForm.weatherStationId.options[theForm.weatherStationId.options.length]=option;
	}
};

/**
 * Start the chain of functions that run the model
 */
var submitForm = function()
{
	// TODO: Validate input
	// Date of sowing format
	if(!getDateOfSowing().isValid())
	{
		alert('Ikke gyldig dato. Tips: Skriv eksempelvis 2014-04-02 for 2. april 2014.');
		return;
	}
	// Date of sowing limit is July 1st
	var thisYear = getDateOfSowing().year();
	var dateOfSowingLimit = moment(thisYear + "-07-01","YYYY-MM-DD");
	if(getDateOfSowing().isAfter(dateOfSowingLimit))
	{
		alert("Saadato er etter 1. juli. Dette stemmer antakelig ikke. Ingen beregninger er foretatt.");
		return;
	}
	var theForm = document.getElementById(theFormId);
	if(theForm.weatherStationId.options[theForm.weatherStationId.selectedIndex].value <= 0)
	{
		alert("Vennligst velg klimastasjon");
		return;
	}
	// See if test input now is set
	if(DEBUG)
	{
		now = getNow();
	}
	
	// Erase previous results
	displayResultHTML("");

	// Start chain of async functions
	createConfig();

}

/**
 * Creates the configuration for weather data. Moves on to running model after data returns
 */
var createConfig = function(){
	
	
	var theForm = document.getElementById(theFormId);
	var weatherStationId = theForm.weatherStationId.options[theForm.weatherStationId.selectedIndex].value;
	var dateOfSowing = getDateOfSowing();
	// Challenge: Get weather data asynchronously first....
	//Tip: Use jQuery.stringify
	//var theConfig = 
	var weatherDataUri = [weatherDataBaseUri,
	                      "?startDate=", dateOfSowing.format("YYYY-MM-DD"),
	                      (DEBUG ? "&today=" + now.format("YYYY-MM-DD") : ""),
	                      "&weatherStationId=", weatherStationId//,
	                      //"&callback=?"
	                      ].join("");
	
	
	var jqxhr = $.ajax( {
		url:weatherDataUri,
		dataType: "json"
		})
	.done(function(data, textStatus, jqXHR) {
		runModel(data);
	})
	.fail(function( jqXHR, textStatus,errorThrown ) {
		alert( "Request failed: " + errorThrown );
	})
	.always(function() {
	//alert( "complete" );
	})
	;
	
		
};

/**
 * Calls the oatFlowering model in the VIPS forecasting system
 * Displays data when results are returned
 */
var runModel = function(data)
{
	//console.log("runModel");
	var dateOfSowing = getDateOfSowing();
	var modelConfig = {
		"loginInfo":{
			"username":"testuser",
			"password":"testpass"
		},
		"modelId":"OATFLOWERM",
		"configParameters":{
			//"dateOfSowing":"2013-04-01T00:00:00+02:00",
			"dateOfSowing":dateOfSowing.format("YYYY-MM-DD"),
			"observations": data
		}
	};

	//console.log(data);
    var request = $.ajax({
        type:"POST",
        url: runUri,
        data: JSON.stringify(modelConfig),
        dataType: "json",
        contentType: "application/json; charset=utf-8",
    })
    .done(function(data, textStatus, jqXHR) {
		displayResults(data);
	})
	.fail(function( jqXHR, textStatus,errorThrown ) {
		alert( "Request failed: " + errorThrown );
	})
	.always(function() {
	//alert( "complete" );
	})
	;
	
}

/**
 * Analyze and display results from the OatFlowering model in VIPS
 */
var displayResults = function(data)
{
	// getting the dates
	var dateOfSowing = getDateOfSowing();
	var dateZ60 = getDateForZ(data,60);
	var weekZ625 = getWeekForZ(data,62.5);
	var dateZ69 = getDateForZ(data,69);
	now = getNow();
	var weekNow = now.week();
	var weeksFromNowToZ625 = Math.abs(weekZ625-weekNow);
	/*console.log("now: " + now.tz("Europe/Oslo").format());
	console.log("z60: " + dateZ60.tz("Europe/Oslo").format());
	console.log("z62.5: " + getDateForZ(data,62.5).tz("Europe/Oslo").format());
	console.log("z69: " + dateZ69.tz("Europe/Oslo").format());*/
	
	
	// Scenario 1: Today is before date of sowing
	if(now.isBefore(dateOfSowing))
	{
		displayResultHTML( ["Havre s&aring;dd ", dateOfSowing.tz("Europe/Oslo").format("YYYY-MM-DD"),
		              " antas &aring; v&aelig;re i blomstring i uke ", weekZ625, " (om ", weeksFromNowToZ625, " uker). ",
		              "Vi gj&oslash;r oppmerksom p&aring; at oppgitt s&aring;dato er frem i tid. ",
		              "Beregningen vil derfor kun v&aelig;re basert p&aring; prognoser og historiske v&aelig;rdata. ",
		              "Dette anslaget vil ha st&oslash;rre grad av sikkerhet n&aring;r dagens dato n&aelig;rmer seg ",
		              "tidspunktet for antatt blomstring.<br/>",
			             (DEBUG ? getRawResults(data) : "")
		              ].join("")
		);
	}
	// Scenario 2: Today is after date of sowing, but more than 10 days before z60
	else if(moment(now).add("days",10).isBefore(dateZ60))
	{
		displayResultHTML( ["Havre s&aring;dd ", dateOfSowing.tz("Europe/Oslo").format("YYYY-MM-DD"),
		              " antas &aring; v&aelig;re i blomstring i uke ", weekZ625, " (om ", weeksFromNowToZ625, " uker). ",
		              "Vi gj&oslash;r oppmerksom p&aring; at beregningen er basert p&aring; observerte v&aelig;rdata, ",
		              "i tillegg til prognoser og historiske v&aelig;rdata. ",
		              "Dette anslaget vil ha st&oslash;rre grad av sikkerhet n&aring;r dagens dato n&aelig;rmer seg ",
		              "tidspunktet for antatt blomstring.<br/>",
		              (DEBUG ? getRawResults(data) : "")
		              ].join("")
		);
	}
	// Scenario 3: Today is after date of sowing, and 0-10 days before z60 OR before z69
	else if((now.isBefore(dateZ60) && moment(now).add("days",11).isAfter(dateZ60)) || now.isBefore(dateZ69))
	{
		displayResultHTML(["Havre s&aring;dd ", dateOfSowing.tz("Europe/Oslo").format("YYYY-MM-DD"),
		              " antas &aring; v&aelig;re i blomstring i perioden fra ", 
		              moment(dateZ60).add("days",-5).tz("Europe/Oslo").format("YYYY-MM-DD"), 
		              " til ",
		              moment(dateZ60).add("days",5).tz("Europe/Oslo").format("YYYY-MM-DD"),
		              ,". ",
		              "Dersom v&aelig;rforholdene ligger til rette for utvikling av mykotoksiner (DON)*, ",
		              "kan fungicidbehandling vurderes foretatt i perioden fra ",dateZ60.tz("Europe/Oslo").format("YYYY-MM-DD"),
		              " og en uke fremover i tid. Beregningen er basert p&aring; observerte v&aelig;rdata ",
		              "i tillegg til prognoser. ",
		              (DEBUG ? getRawResults(data) : "")
		              ].join("")
		);
	}
	
	// Scenario 4: Today is after z69. Too late
	else 
	{
		displayResultHTML(["Havre s&aring;dd ", dateOfSowing.tz("Europe/Oslo").format("YYYY-MM-DD"),
		             " antas &aring; ha v&aelig;rt i blomstring i uke ", weekZ625, " (for ", weeksFromNowToZ625, " uker siden). ",
		             "Det er antakelig for sent &aring; behandle med fungicid for &aring; redusere risikoen ",
		             "for utvikling av DON.<br/>",
		             (DEBUG ? getRawResults(data) : "")
		             ].join("")
		             );
	}
	//console.log(data);
}

/**
 * Util/DEBUG function for displaying the results returned from VIPS
 */
var getRawResults = function(data)
{
	var result = "<h2>R&aring;data</h2><ul>";
	for(var key in data)
	{
		result +="<li>";
		result += "z" + data[key].allValues["OATFLOWERM.ZREACHED"] + " n&aring;dd  " 
			+ moment(data[key].validTimeStart).tz("Europe/Oslo").format("YYYY-MM-DD")
			+ " (uke " + data[key].allValues["OATFLOWERM.WEEK_IN_YEAR"] + ")<br/>";
		result +="</li>";
	}
	result +="</ul>";
	return result;
}

/**
 * Interface for displaing the results
 */
var displayResultHTML = function(text)
{
	document.getElementById("oatFloweringModelResults").innerHTML= "<h2>Resultater</h2>" + text;
}

/**
 * Loops through results from VIPS, returns date for stadium Z
 * Possible values for Z are [60,62.5,69]
 */
var getDateForZ = function(data,z)
{
	for(var key in data)
	{
		if(data[key].allValues["OATFLOWERM.ZREACHED"] == z)
		{
			return moment(data[key].validTimeStart);
		}
	}
}

/**
 * Loops through results from VIPS, returns week number (in year) for stadium Z
 * Possible values for Z are [60,62.5,69]
 */
var getWeekForZ = function(data,z)
{
	for(var key in data)
	{
		if(data[key].allValues["OATFLOWERM.ZREACHED"] == z)
		{
			return data[key].allValues["OATFLOWERM.WEEK_IN_YEAR"];
		}
	}
}

/**
 * Get date of sowing from form
 */
var getDateOfSowing = function()
{
	var theForm = document.getElementById(theFormId);
	var dateOfSowing = moment(theForm.dateOfSowing.value,"YYYY-MM-DD");
	return dateOfSowing;
}

/**
 * Util function for getting current time
 * @returns
 */
function getNow()
{
	var theForm = document.getElementById(theFormId);
	if(theForm.now != undefined)
	{
		var formNow = moment(theForm.now.value,"YYYY-MM-DD");
		return formNow.isValid() ? formNow : now;
	}
	else return now;
}

/**
 * @return the HTML for the form
 */
var getStartHTML = function()
{
	return [
	        	 
                 "<form id='",theFormId,"'>",
                 "<label for='dateOfSowing'>S&aring;dato (&Aring;&Aring;&Aring;&Aring;-MM-DD)</label> ",
                 "<input type='text' name='dateOfSowing', value='",now.format("YYYY"),"-0X-XX'/><br/>",
                 (DEBUG ? "<label for='now'>N&aring;dato (&Aring;&Aring;&Aring;&Aring;-MM-DD)</label>" : ""),
                 (DEBUG ? ["<input type='text' name='now' value='", now.format("YYYY-MM-DD"), "'/><br/>"].join("") : ""),
                 "Klimastasjon <select name='weatherStationId'>",
                 "<option value='-1'>-- Vennligst velg klimastasjon --</option>",
                 "</select><br/>",
                 "<button type='button' onclick='submitForm();'>Beregn</button>",
                 "</form>",
                 "<div id='resultsTable'></div>"
                 ].join("");
}
