#
# Copyright (c) 2013-2023 NIBIO.
#
# This file is part of VIPSWeb 
# (see https://gitlab.nibio.no/VIPS/VIPSWeb).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#

from datetime import datetime
from dateutil.relativedelta import relativedelta

from django.http import HttpResponse
from django.shortcuts import render
from django.conf import settings
from django.utils import translation

from forecasts.models import ForecastConfiguration, ForecastResult, ResultParameter, ResultParameterLocal, Model, MeasurementUnit, ModelGraphParameter

def index(request):
    
    season_range = range(2016, datetime.now().year + 1)
    try:
        season = int(request.GET.get("season", (datetime.now() + relativedelta(months = settings.SYSTEM_TIME_OFFSET_MONTHS)).year))
    except ValueError:
        season = (datetime.now() + relativedelta(months = settings.SYSTEM_TIME_OFFSET_MONTHS)).year
    forecast_configurations = ForecastConfiguration.get_forecast_configurations_from_vipslogic(None, season).text
    #print forecast_configurations
    private_forecast_configurations = None
    if request.session.get("user_uuid",None) != None:
        private_forecast_configurations = ForecastConfiguration.get_private_forecast_configurations(request.session["user_uuid"])
    #forecast_configurations.sort(key=lambda x: x.date_start, reverse=False)
    context = {
               'forecast_configurations': forecast_configurations,
               'private_forecast_configurations': private_forecast_configurations,
               'models_local_names' : Model.get_models_local_names().text,
               'season': season,
               'season_range': season_range
               }
    return render(request, 'forecasts/index.html', context)

def detail_latest_days(request, forecast_id, latest_days):
    forecast_configuration = ForecastConfiguration.get_forecast_configuration(forecast_id, request.session.get("user_uuid",None))
    if forecast_configuration == None:
        return render(request, 'forecasts/detail_error.html')
    #start = time.time()
    forecast_results = ForecastResult.get_forecast_results(forecast_id, request.session.get("user_uuid",None), latest_days)
    #forecast_results = forecast_results[0:1500]
    forecast_result_highcharts = ForecastResult.get_forecast_results_highcharts(forecast_results, forecast_id, request.session.get("user_uuid",None))
    forecast_warning_statuses_highcharts = ForecastResult.get_forecast_warning_statuses_highcharts(forecast_results)
    #print "Result: %s seconds" % (time.time() - start) 
    result_parameters = None
    if len(forecast_results) > 0:
        all_result_keys = []
        for forecast_result in forecast_results:
            for key in forecast_result.all_values.keys():
                if key not in all_result_keys:
                    all_result_keys.append(key)
        inStr = "'" + "','".join(all_result_keys) + "'"
        result_parameters_temp = ResultParameter.objects.raw("SELECT * FROM forecasts_resultparameter WHERE namespace || '.' || key IN (%s)" % inStr)
        result_parameters = []
        for p in result_parameters_temp:
            result_parameter_local_set = ResultParameterLocal.objects.filter(result_parameter_id=p.id)
            if len(result_parameter_local_set) > 0:
                for language_code in [translation.get_language(), settings.LANGUAGE_CODE,'en']:
                    for result_parameter_local in result_parameter_local_set:
                        if result_parameter_local.language_code == language_code:
                            p.set_local_name(result_parameter_local.local_name)
            result_parameters.append(p)

    context = {
               'forecast_configuration': forecast_configuration,
               'latest_days': latest_days,
               'result_parameters' : result_parameters,
               'forecast_results': forecast_results,
               'forecast_result_highcharts' : forecast_result_highcharts,
               'forecast_warning_statuses_highcharts' : forecast_warning_statuses_highcharts,
               'timeseriesURL': '/forecasts/%s/detail_highcharts_json/' % forecast_id
               }
    return render(request, 'forecasts/detail.html', context)

def detail(request, forecast_id):
    return detail_latest_days(request,forecast_id,30)
    

def models_index(request):
    return HttpResponse("This is the models index page. More to come!")

def models_detail(request, model_id):
    model = Model.get_model(model_id)
    context = {'model': model}
    return render(request, 'models/detail.html', context)

def models_detail_test(request):
    context = {}
    return render(request, 'models/detail_test.html', context)

# Serving models' local names for the forecast Map
def model_local_names_js(request):
    models_local_names = Model.get_models_local_names_as_json()
    #print models_local_names
    context = {'models_local_names': models_local_names}
    return render(request,'models/modelLocalNames.js', context, content_type="application/javascript;charset=UTF-8")

# Returning results as Highcharts timeseries (JSON)
def detail_highcharts_json(request, forecast_id):
    forecast_results = ForecastResult.get_forecast_results(forecast_id)
    
    the_response = ForecastResult.get_forecast_results_highcharts(forecast_results, forecast_id);
    #print "Result 6: %s seconds from start" % (time.time() - start) 
    
    return HttpResponse(the_response, content_type='application/json')


    
    